"""NBA games table"""

from .dynamodb_table import DynamoDBTable
from ..models import NbaPlayerPerformanceModel

class NbaPlayersStatsTable(DynamoDBTable[NbaPlayerPerformanceModel]):
    def __init__(self, environment: str):
        DynamoDBTable.__init__(self, 'nba-players-stats', environment, 'playerId', 'gameDate')

class NbaPlayerPerformancesTable(DynamoDBTable[NbaPlayerPerformanceModel]):
    def __init__(self, environment: str):
        DynamoDBTable.__init__(self, 'nba-players-stats', environment, 'playerId', 'gameDate')



"""NBA game status"""

from enum import Enum

class NbaGameExtendedStatus(str, Enum):
    REGULAR = 'regular'
    POSTPONED = 'postponed'

    @classmethod
    def match(cls, status_number: int) -> 'NbaGameExtendedStatus':
        if status_number == 0: return cls.REGULAR
        elif status_number == 2: return cls.POSTPONED
        return cls.REGULAR



"""Week id module"""

from typing import List, Type
from datetime import datetime

from isoweek import Week

from ..helpers.date_helper import DateHelper
from .basketix_season import BasketixSeason

class BasketixWeek:
    """Basketix week class"""

    SEPARATOR = '_'

    def __init__(self, value: str) -> None:
        self._value = value

    def __repr__(self) -> str:
        return f'Basketix week: {self._value}'

    @property
    def id(self) -> str:
        """Returns week id"""

        return self._value

    @property
    def year(self) -> int:
        """Returns week year"""

        return int(self._value.split(self.SEPARATOR)[0])

    @property
    def week_number(self) -> int:
        """Returns week number"""

        return int(self._value.split(self.SEPARATOR)[1])

    @property
    def days(self) -> List[str]:
        """Returns all days of a week."""

        return [day.strftime(DateHelper.DATE_ISO_FORMAT) for day in Week(self.year, self.week_number).days()]

    @property
    def first_day(self) -> str:
        """Returns first days of a week as string"""

        return self.days[0]

    @property
    def last_day(self) -> str:
        """Returns last days of a week as string"""

        return self.days[-1]

    @property
    def season(self) -> 'BasketixSeason':
        """Returns all days of a week."""

        first_day_of_week = self.days[0]

        return BasketixSeason.from_date(first_day_of_week)

    def next(self, delta=1) -> 'BasketixWeek':
        """Returns next delta week"""

        date = DateHelper.delta_days(self.first_day, days=7*delta)

        return self.from_date(date=date)

    def previous(self, delta=1) -> 'BasketixWeek':
        """Returns previous delta week"""

        date = DateHelper.delta_days(self.first_day, days=-7*delta)

        return self.from_date(date=date)

    @classmethod
    def from_date(cls: Type['BasketixWeek'], date: str) -> 'BasketixWeek':
        """Parses date and returns Week instance"""

        parse_date = datetime.strptime(date, DateHelper.DATE_ISO_FORMAT)
        year, week_number, _ = parse_date.isocalendar()

        week_id = f'{year}_{week_number:02d}'

        return cls(value=week_id)



'''Transactions entities module'''

class Transactions():
    GAME_CONFIG = 'GameConfig'
    DRAFT_PICK = 'DraftPick'
    FREE_AGENCY = 'FreeAgency'

    #Free Agency
    BID = 'FreeAgency_Bid'
    BID_DELETE = 'FreeAgency_BidDelete'
    BID_LOOSE = 'FreeAgency_loose'
    BID_WIN = 'FreeAgency_win'
    CUT = 'FreeAgency_Cut'



from typing import Dict, List, TypedDict
from typing_extensions import NotRequired

from .attributes.basketix_team_id import BasketixTeamId
from .attributes.game_id import GameId
from .attributes.nba_team_id import NbaTeamId
from .attributes.player_id import PlayerId
from .attributes.positions import Position
from .attributes.season_id import SeasonId
from .attributes.week_id import WeekId

class PlayerBonus(TypedDict):
    playerId: NotRequired[PlayerId]
    points: int

class BestPlayer(PlayerBonus, total=False):
    totalEvaluation: int

class BestGame(PlayerBonus, total=False):
    evaluation: int
    gameId: GameId

class Evaluation(TypedDict):
    playerId: PlayerId
    gameId: GameId
    teamId: NbaTeamId
    gameDate: str
    evaluation: int

class Players(TypedDict):
    playerId: PlayerId
    evaluation: int
    evaluations: List[Evaluation]
    positions: List[Position]

class Bonus(TypedDict):
    bestGame: BestGame
    bestPlayer: BestPlayer
    points: int

class Points(TypedDict):
    bonus: Bonus
    rank: int
    total: int

class Ranks(TypedDict):
    evaluation: int
    final: int

class BasketixTeamsModel(TypedDict):
    '''Model of the basketix teams table'''
    weekId: WeekId
    teamId: BasketixTeamId
    basketixId: BasketixTeamId
    seasonId: SeasonId
    isValidate: bool
    validateAt: str
    isEditable: bool
    points: Points
    ranks: Ranks
    evaluation: int
    players: Dict[PlayerId, Players]
    starters: Dict[Position, PlayerId]
    initialStarters: Dict[Position, PlayerId]
    substitutes: List[PlayerId]
    initialSubstitutes: List[PlayerId]



'''League status entities module'''

TRADUCTION = {
    'WAITING': {
        'en': 'Waiting',
        'fr': 'En attente'
    },
    'DRAFT': {
        'en': 'Draft',
        'fr': 'Draft'
    },
    'REGULARSEASON': {
        'en': 'Regular season',
        'fr': 'Saison régulière'
    },
    'FINISHED': {
        'en': 'Finished',
        'fr': 'Terminée'
    },
}

class LeagueStatus():
    '''League status'''

    WAITING = 'WAITING'
    DRAFT = 'DRAFT'
    REGULAR_SEASON = 'REGULARSEASON'
    FINISHED = 'FINISHED'

    @classmethod
    def translate(cls, status: str, language: str = 'en'):
        '''Translate a league status'''
        return TRADUCTION[status][language]



"""NBA player performance model"""

from typing import TypedDict

from .nba_game_status import NbaGameStatus

class NbaPlayerPerformanceModel(TypedDict):
    """DynamoDB model for NBA player performance"""
    gameId: str
    weekId: str
    season: str
    gameDate: str
    gameStatus: NbaGameStatus
    playerId: str
    firstName: str
    lastName: str
    fullName: str
    teamId: str
    teamTriCode: str
    teamName: str
    teamCity: str
    teamFullName: str
    points: int
    sec: int
    fgm: int
    fga: int
    fgpct: float
    ftm: int
    fta: int
    ftpct: float
    tpm: int
    tpa: int
    tppct: float
    offReb: int
    defReb: int
    totReb: int
    assists: int
    pFouls: int
    steals: int
    turnovers: int
    blocks: int
    plusMinus: int
    evaluation: int



from .basketix_teams_model import BasketixTeamsModel, Evaluation, Players

from .nba_players_performances import NbaPlayerPerformanceModel
from .nba_games import NbaGameModel
from .nba_game_status import NbaGameStatus
from .nba_game_extended_status import NbaGameExtendedStatus
from .nba_game_location import NbaGameLocation
from .nba_players import NbaPlayerModel, NbaPlayerInjuryInfo, NbaPlayerTeamInfo, NbaPlayerStatInfo, NbaPlayerEvaluationInfo

from .attributes.positions import Position, Position_Enum
from .attributes.player_id import PlayerId



"""NBA player model"""

from typing import TypedDict, List

from typing_extensions import NotRequired

from .attributes.positions import Position
from .attributes.rotoworld_id import RotoworldId

class NbaPlayerTeamInfo(TypedDict):
    """Sub model for team info for NBA player model"""
    teamId: str
    tricode: str
    city: str
    nickname: str
    fullName: str

NbaPlayerInjuryInfo = TypedDict('NbaPlayerInjuryInfo', {
    'return': str,
    'type': str,
    'date': str,
})

class NbaPlayerEvaluationInfo(TypedDict):
    """Sub model for stat info for NBA player model"""
    gameDate: str
    gameId: str
    weekId: str
    evaluation: int

class NbaPlayerStatInfo(TypedDict):
    """Sub model for stat info for NBA player model"""
    computationDate: int
    n: int
    nPlayed: int
    pctPlayed: int
    evaluationSum: int
    evaluationMean: float
    evaluationStd: int
    evaluationMin: int
    evaluationMax: int
    lastEvaluations: List[NbaPlayerEvaluationInfo]

class NbaPlayerModel(TypedDict):
    """DynamoDB model for NBA player"""
    playerId: str
    fullName: str
    firstName: str
    lastName: str
    teamId: NotRequired[str]
    team: NotRequired[NbaPlayerTeamInfo]
    teamDate: NotRequired[str]
    positions: NotRequired[List[Position]]
    stats: NotRequired[NbaPlayerStatInfo]
    cost: NotRequired[int]
    costComputationDate: NotRequired[str]
    injury: NotRequired[NbaPlayerInjuryInfo]
    rotoworldId: NotRequired[RotoworldId]



"""Basketix leagues table"""

from .dynamodb_table import DynamoDBTable

class BasketixLeaguesTable(DynamoDBTable):
    def __init__(self, environment):
        DynamoDBTable.__init__(self, 'basketix-leagues', environment, 'leagueId')



"""NBA games table"""

from .dynamodb_table import DynamoDBTable
from ..models import NbaGameModel

class NbaGamesTable(DynamoDBTable[NbaGameModel]):
    def __init__(self, environment: str):
        DynamoDBTable.__init__(self, 'nba-games', environment, 'gameId')



"""NBA teams module"""

from ..exceptions import UnknownNBATeam


class NbaTeamsHandler:
    """Handle NBA teams."""

    _MAPPING_ID_TRICODE = {
        "1610612737": "ATL",
        "1610612738": "BOS",
        "1610612739": "CLE",
        "1610612740": "NOP",
        "1610612741": "CHI",
        "1610612742": "DAL",
        "1610612743": "DEN",
        "1610612744": "GSW",
        "1610612745": "HOU",
        "1610612746": "LAC",
        "1610612747": "LAL",
        "1610612748": "MIA",
        "1610612749": "MIL",
        "1610612750": "MIN",
        "1610612751": "BKN",
        "1610612752": "NYK",
        "1610612753": "ORL",
        "1610612754": "IND",
        "1610612755": "PHI",
        "1610612756": "PHX",
        "1610612757": "POR",
        "1610612758": "SAC",
        "1610612759": "SAS",
        "1610612760": "OKC",
        "1610612761": "TOR",
        "1610612762": "UTA",
        "1610612763": "MEM",
        "1610612764": "WAS",
        "1610612765": "DET",
        "1610612766": "CHA",
    }

    _MAPPING_TRICODE_ID = {tricode: team_id for team_id, tricode in _MAPPING_ID_TRICODE.items()}

    @classmethod
    def tricode(cls, team_id: str):
        """
        Returns tricode from an ID.

        Raises UnknownTeam error if team_id is unknown.
        """
        try:
            return cls._MAPPING_ID_TRICODE[team_id]
        except KeyError as err:
            raise UnknownNBATeam(team_id=team_id) from err

    @classmethod
    def id(cls, tricode: str):
        """
        Returns ID from a tricode.

        Raises UnknownTeam error if tricode is unknown.
        """
        try:
            return cls._MAPPING_TRICODE_ID[tricode.upper()]
        except KeyError as err:
            raise UnknownNBATeam(tricode=tricode.upper()) from err
